package de.uulm.omi.dbms.workload.api.impl;

import de.uulm.omi.cloudiator.dbms.evaluator.model.ProcessStatus;
import de.uulm.omi.cloudiator.dbms.evaluator.model.TPCCWorkload;
import de.uulm.omi.cloudiator.dbms.evaluator.model.TSBSWorkload;
import de.uulm.omi.cloudiator.dbms.evaluator.model.WorkloadProcessInstance;
import de.uulm.omi.cloudiator.dbms.evaluator.model.WorkloadProcessSubmissionError;
import de.uulm.omi.cloudiator.dbms.evaluator.model.YCSBWorkload;
import de.uulm.omi.dbms.workload.api.ApiResponseMessage;
import de.uulm.omi.dbms.workload.api.NotFoundException;
import de.uulm.omi.dbms.workload.api.WorkloadApiService;
import de.uulm.omi.utils.TextFileLoader;
import de.uulm.omi.utils.WorkloadApiProperties;
import de.uulm.omi.utils.WorkloadProcessChecker;
import de.uulm.omi.workload.WorkloadProcess;
import de.uulm.omi.workload.tpcc.TPCCCommandBuilder;
import de.uulm.omi.workload.tsbs.TSBSCommandBuilder;
import de.uulm.omi.workload.ycsb.YCSBCommandBuilder;
import java.io.File;
import java.io.FileNotFoundException;
import java.math.BigDecimal;
import java.util.Properties;
import javax.validation.constraints.NotNull;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.Response.Status;
import javax.ws.rs.core.SecurityContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

@javax.annotation.Generated(value = "io.swagger.codegen.languages.JavaJerseyServerCodegen", date = "2018-01-16T09:59:22.348Z")
public class WorkloadApiServiceImpl extends WorkloadApiService {

  private final Logger LOGGER = LoggerFactory.getLogger(this.getClass());

  private final String WINDOWS_SHELL = "cmd.exe";
  private final String WINDOWS_COMMAND_PARAM = "/c";

  private final String UNIX_SHELL = "bash";
  private final String UNIX_COMMAND_PARAM = "-c";

  //YCSB constants
  private final String YCSB_RESULT_FOLDER = "YCSB/";
  private static final String YCSB_BINARY = "bin/ycsb";
  private static final String YCSB_FOLDER = "ycsb";

  //TPC-C constants
  private final String TPCC_RESULT_FOLDER = "TPCC/";
  private static final String TPCC_BINARY = "./cockroach";

  //TSBS constants
  private final String TSBS_RESULT_FOLDER = "TSBS/";
  private static final String TSBS_BINARY = "/opt/workloads/tsbs/bin/";


  @Override
  public Response workloadResultGet(@NotNull String taskId, @NotNull String workloadType,
      SecurityContext securityContext) throws NotFoundException {

    File resultFile = null;
    try {
      resultFile = TextFileLoader.loadScript(taskId, workloadType);
    } catch (FileNotFoundException e) {
      LOGGER.error("Error while loading result file, returning empty file", e);
      return Response.serverError()
          .entity(new ApiResponseMessage(ApiResponseMessage.ERROR, "Unable to load result file!"))
          .build();
    }

    return Response.ok().entity(resultFile).build();

  }

  @Override
  public Response workloadStatusGet(String applicationInstanceId, SecurityContext securityContext)
      throws NotFoundException {

    //TODO: remove applicationInstanceId from call is the parameter is obsolote!

    WorkloadProcess workloadProcess = WorkloadProcess.getInstance();
    WorkloadProcessInstance workloadProcessInstance = new WorkloadProcessInstance();
    workloadProcessInstance.setTimestamp(BigDecimal.valueOf(workloadProcess.getStartTimestamp()));
    workloadProcessInstance.setProcessId(workloadProcess.getProcessId());
    if (workloadProcess.isRunning()) {
      workloadProcessInstance.setProcessStatus(ProcessStatus.RUNNING);
      return Response.ok().entity(workloadProcessInstance).build();
    } else {
      if (workloadProcess.getExitCode() != 0) {
        workloadProcessInstance.setProcessStatus(ProcessStatus.ERROR);
        return Response.ok().entity(workloadProcessInstance).build();
      } else {
        workloadProcessInstance.setProcessStatus(ProcessStatus.IDLE);
        return Response.ok().entity(workloadProcessInstance).build();
      }
    }

  }

  @Override
  public Response workloadTpccPost(@NotNull String taskId,
      TPCCWorkload workload, SecurityContext securityContext) throws NotFoundException {



    //check if already a workloadProcess running
    WorkloadProcess workloadProcess = WorkloadProcess.getInstance();
    if (workloadProcess.isRunning()) {
      WorkloadProcessSubmissionError workloadProcessSubmissionError = new WorkloadProcessSubmissionError();
      workloadProcessSubmissionError.setMessage(
          "There is already a workloadProcess running, please wait until workloadProcess is finished!");

      return Response.status(Status.FORBIDDEN).entity(workloadProcessSubmissionError).build();
    }

    Properties properties = WorkloadApiProperties.getInstance().getProperties();

    String binariyDirectory =
        properties.getProperty(WorkloadApiProperties.WORKLOAD_BINARIES_FOLDER) + "cockroach/binary";

    String resultFileName =
        properties.getProperty(WorkloadApiProperties.WORKLOAD_RESULTS_FOLDER) + TPCC_RESULT_FOLDER
            + taskId + ".txt";

    WorkloadProcessInstance workloadProcessInstance = new WorkloadProcessInstance();

    /*
    //TODO: use this code for the future with an explicit init phase
    //Init the workload

    Long workloadTimestamp = System.currentTimeMillis();
    LOGGER.debug("Creating TPC-C init command...");
    String initCommand = TPCCCommandBuilder.buildInitCommand(workload.getDbEndpoints(),TPCC_BINARY);
    initCommand += " &> " + resultFileName;

    workloadProcess
        .startWorkload(binariyDirectory, UNIX_SHELL, UNIX_COMMAND_PARAM, initCommand, taskId,
            workloadTimestamp);



    workloadProcessInstance.setTimestamp(BigDecimal.valueOf(workloadProcess.getStartTimestamp()));
    workloadProcessInstance.setProcessId(workloadProcess.getProcessId());

    if (workloadProcess.isRunning()) {
      LOGGER.debug("WorkloadPrcess started!");
      workloadProcessInstance.setProcessStatus(ProcessStatus.RUNNING);
    } else {
      if (workloadProcess.getExitCode() != 0) {
        LOGGER.debug("workloadPrcess exited with " + workloadProcess.getExitCode()
            + " Incorrect command provided? Command: " + initCommand);
        workloadProcessInstance.setProcessStatus(ProcessStatus.ERROR);
        return Response.ok().entity(workloadProcessInstance).build();
      } else {
        LOGGER.debug(
            "workloadPrcess already finished, incorrect command provided? Command: " + initCommand);
        workloadProcessInstance.setProcessStatus(ProcessStatus.IDLE);
        return Response.ok().entity(workloadProcessInstance).build();
      }
    }

    // wait until init process is finished
    while (workloadProcess.isRunning()){
      try {
        LOGGER.debug("Waiting for TPC-C init to be finished, checking again in 5s...");
        Thread.sleep(5000);
      } catch (InterruptedException e) {
        e.printStackTrace();
      }
    }

    LOGGER.debug("TPC-C init finished, now starting the TPC-C execution!");


     */

    //Run the workload
    Long workloadTimestamp = System.currentTimeMillis();

    String startCommand = "";
    startCommand = TPCCCommandBuilder.buildStartCommand(workload.getTpccWorkloadConfig(), workload.getDbEndpoints(), TPCC_BINARY);
    startCommand += " &> " + resultFileName;


    workloadProcess
        .startWorkload(binariyDirectory, UNIX_SHELL, UNIX_COMMAND_PARAM, startCommand, taskId,
            workloadTimestamp);
    workloadProcessInstance = new WorkloadProcessInstance();
    workloadProcessInstance.setTimestamp(BigDecimal.valueOf(workloadProcess.getStartTimestamp()));
    workloadProcessInstance.setProcessId(workloadProcess.getProcessId());
    if (workloadProcess.isRunning()) {
      LOGGER.debug("WorkloadPrcess started!");
      workloadProcessInstance.setProcessStatus(ProcessStatus.RUNNING);
      return Response.ok().entity(workloadProcessInstance).build();
    } else {
      if (workloadProcess.getExitCode() != 0) {
        LOGGER.debug("workloadPrcess exited with " + workloadProcess.getExitCode()
            + " Incorrect command provided? Command: " + startCommand);
        workloadProcessInstance.setProcessStatus(ProcessStatus.ERROR);
        return Response.ok().entity(workloadProcessInstance).build();
      } else {
        LOGGER.debug(
            "workloadPrcess already finished, incorrect command provided? Command: " + startCommand);
        workloadProcessInstance.setProcessStatus(ProcessStatus.IDLE);
        return Response.ok().entity(workloadProcessInstance).build();
      }
    }

  }

  @Override
  public Response workloadTsbsExecuteQueriesPost(String taskId, TSBSWorkload workload,
      SecurityContext securityContext) throws NotFoundException {

    if(WorkloadProcessChecker.checkWorkloadProcessStatee() != null){
      return WorkloadProcessChecker.checkWorkloadProcessStatee();
    }

    WorkloadProcess workloadProcess = WorkloadProcess.getInstance();

    Properties properties = WorkloadApiProperties.getInstance().getProperties();
    Long workloadTimestamp = System.currentTimeMillis();


    String executeQueriesCommand = TSBSCommandBuilder.executeQueriesCommand(TSBS_BINARY, properties.getProperty(WorkloadApiProperties.WORKLOAD_DATA_FOLDER), workload.getDbms(),   workload.getTsbsQueryExecution(), workload.getDbEndpoints() );
    executeQueriesCommand += " &> "  + properties.getProperty(WorkloadApiProperties.WORKLOAD_RESULTS_FOLDER) +  TSBS_RESULT_FOLDER + taskId.trim() + ".txt";


    workloadProcess
        .startWorkload(TSBS_BINARY, UNIX_SHELL, UNIX_COMMAND_PARAM, executeQueriesCommand, taskId,
            workloadTimestamp);

    WorkloadProcessInstance workloadProcessInstance = new WorkloadProcessInstance();
    workloadProcessInstance.setProcessStatus(ProcessStatus.RUNNING);
    workloadProcessInstance.setTimestamp(BigDecimal.valueOf(workloadProcess.getStartTimestamp()));
    workloadProcessInstance.setProcessId(workloadProcess.getProcessId());


    return Response.ok().entity(workloadProcessInstance).build();
  }

  @Override
  public Response workloadTsbsGenerateDataPost(String taskId, TSBSWorkload workload,
      SecurityContext securityContext) throws NotFoundException {

    if(WorkloadProcessChecker.checkWorkloadProcessStatee() != null){
      return WorkloadProcessChecker.checkWorkloadProcessStatee();
    }

    WorkloadProcess workloadProcess = WorkloadProcess.getInstance();

    Properties properties = WorkloadApiProperties.getInstance().getProperties();
    Long workloadTimestamp = System.currentTimeMillis();

    LOGGER.debug(workload.toString());

    //Phase 1: generate data
    LOGGER.debug("Starting phase 1: data generations");
    String generateDataCommand= "";
    generateDataCommand = TSBSCommandBuilder
        .generateDataCommand( TSBS_BINARY, workload.getDbms(), workload.getTsbsDataGeneration());
    generateDataCommand += properties.getProperty(WorkloadApiProperties.WORKLOAD_DATA_FOLDER) + workload.getDbms().toString().toLowerCase() + "-data.gz";


    workloadProcess
        .startWorkload(TSBS_BINARY, UNIX_SHELL, UNIX_COMMAND_PARAM, generateDataCommand, taskId,
            workloadTimestamp);

    WorkloadProcessInstance workloadProcessInstance = new WorkloadProcessInstance();
    workloadProcessInstance.setProcessStatus(ProcessStatus.RUNNING);
    workloadProcessInstance.setTimestamp(BigDecimal.valueOf(workloadProcess.getStartTimestamp()));
    workloadProcessInstance.setProcessId(workloadProcess.getProcessId());


    return Response.ok().entity(workloadProcessInstance).build();
  }

  @Override
  public Response workloadTsbsGenerateQueriesPost(String taskId, TSBSWorkload workload,
      SecurityContext securityContext) throws NotFoundException {

    if(WorkloadProcessChecker.checkWorkloadProcessStatee() != null){
      return WorkloadProcessChecker.checkWorkloadProcessStatee();
    }

    WorkloadProcess workloadProcess = WorkloadProcess.getInstance();

    Properties properties = WorkloadApiProperties.getInstance().getProperties();
    Long workloadTimestamp = System.currentTimeMillis();


    String generateQueriesCommand = "";
    generateQueriesCommand = TSBSCommandBuilder.generateQueriesCommand(TSBS_BINARY, workload.getDbms(), workload.getTsbsDataGeneration(), workload.getTsbsQueryGeneration());
    generateQueriesCommand += properties.getProperty(WorkloadApiProperties.WORKLOAD_DATA_FOLDER) + workload.getDbms().toString().toLowerCase() + "-queries.gz" ;

    workloadProcess
        .startWorkload(TSBS_BINARY, UNIX_SHELL, UNIX_COMMAND_PARAM, generateQueriesCommand, taskId,
            workloadTimestamp);

    WorkloadProcessInstance workloadProcessInstance = new WorkloadProcessInstance();
    workloadProcessInstance.setProcessStatus(ProcessStatus.RUNNING);
    workloadProcessInstance.setTimestamp(BigDecimal.valueOf(workloadProcess.getStartTimestamp()));
    workloadProcessInstance.setProcessId(workloadProcess.getProcessId());


    return Response.ok().entity(workloadProcessInstance).build();
  }

  @Override
  public Response workloadTsbsLoadDataPost(String taskId, TSBSWorkload workload,
      SecurityContext securityContext) throws NotFoundException {

    if(WorkloadProcessChecker.checkWorkloadProcessStatee() != null){
      return WorkloadProcessChecker.checkWorkloadProcessStatee();
    }

    WorkloadProcess workloadProcess = WorkloadProcess.getInstance();

    Properties properties = WorkloadApiProperties.getInstance().getProperties();
    Long workloadTimestamp = System.currentTimeMillis();

    String loadCommand = "";
    loadCommand = TSBSCommandBuilder.generateLoadCommand(TSBS_BINARY, properties.getProperty(WorkloadApiProperties.WORKLOAD_DATA_FOLDER), workload.getDbms(), workload.getTsbsLoadExecution(),workload.getDbEndpoints());
    loadCommand += " &> " + properties.getProperty(WorkloadApiProperties.WORKLOAD_RESULTS_FOLDER) + TSBS_RESULT_FOLDER + taskId.trim() + "_load.txt";


    workloadProcess
        .startWorkload(TSBS_BINARY, UNIX_SHELL, UNIX_COMMAND_PARAM, loadCommand, taskId,
            workloadTimestamp);


    WorkloadProcessInstance workloadProcessInstance = new WorkloadProcessInstance();
    workloadProcessInstance.setProcessStatus(ProcessStatus.RUNNING);
    workloadProcessInstance.setTimestamp(BigDecimal.valueOf(workloadProcess.getStartTimestamp()));
    workloadProcessInstance.setProcessId(workloadProcess.getProcessId());


    return Response.ok().entity(workloadProcessInstance).build();
  }


  @Override
  public Response workloadYcsbPost(@NotNull String taskId,
      YCSBWorkload workload, SecurityContext securityContext) throws NotFoundException {

    LOGGER.warn("Custom ports are currently not supported! Default ports of the respective DBMS will be used!");


    //check if already a workloadProcess running
    WorkloadProcess workloadProcess = WorkloadProcess.getInstance();
    if (workloadProcess.isRunning()) {
      WorkloadProcessSubmissionError workloadProcessSubmissionError = new WorkloadProcessSubmissionError();
      workloadProcessSubmissionError.setMessage(
          "There is already a workloadProcess running, please wait until workloadProcess is finished!");

      return Response.status(Status.FORBIDDEN).entity(workloadProcessSubmissionError).build();
    }

    Properties properties = WorkloadApiProperties.getInstance().getProperties();

    Long workloadTimestamp = System.currentTimeMillis();


    String startCommand = "";
    startCommand = YCSBCommandBuilder.buildStartCommand(workload, YCSB_BINARY);


    String resultFileName =
        properties.getProperty(WorkloadApiProperties.WORKLOAD_RESULTS_FOLDER) + YCSB_RESULT_FOLDER
            + taskId.trim() + ".txt";
    startCommand += " &> " + resultFileName;

    //TODO: get binary and version from API call to add more workloads
    String binariyDirectory =
        properties.getProperty(WorkloadApiProperties.WORKLOAD_BINARIES_FOLDER) + YCSB_FOLDER;
    workloadProcess
        .startWorkload(binariyDirectory, UNIX_SHELL, UNIX_COMMAND_PARAM, startCommand, taskId,
            workloadTimestamp);

    WorkloadProcessInstance workloadProcessInstance = new WorkloadProcessInstance();
    workloadProcessInstance.setTimestamp(BigDecimal.valueOf(workloadProcess.getStartTimestamp()));
    workloadProcessInstance.setProcessId(workloadProcess.getProcessId());
    if (workloadProcess.isRunning()) {
      LOGGER.debug("WorkloadPrcess started!");
      workloadProcessInstance.setProcessStatus(ProcessStatus.RUNNING);
      return Response.ok().entity(workloadProcessInstance).build();
    } else {
      if (workloadProcess.getExitCode() != 0) {
        LOGGER.debug("workloadPrcess exited with " + workloadProcess.getExitCode()
            + " Incorrect command provided? Command: " + "TODO");
        workloadProcessInstance.setProcessStatus(ProcessStatus.ERROR);
        return Response.ok().entity(workloadProcessInstance).build();
      } else {
        LOGGER.debug(
            "workloadPrcess already finished, incorrect command provided? Command: " + "TODO");
        workloadProcessInstance.setProcessStatus(ProcessStatus.IDLE);
        return Response.ok().entity(workloadProcessInstance).build();
      }
    }


  }

}
