package de.uulm.omi.utils;

import static com.google.common.base.Preconditions.checkNotNull;

import de.uulm.omi.workload.WorkloadProcess;
import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;
import javax.servlet.ServletContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Created by Daniel Seybold on 16.01.2018.
 */
public enum WorkloadApiProperties implements WorkloadApiPropertiesService {


  INSTANCE{

    Properties properties;

    @Override
    public void initProperties(ServletContext context) {

      checkNotNull(context);

      properties = new Properties();

      InputStream input = null;

      try {

        input =  context.getResourceAsStream(PROPERTIES_PATH);
        checkNotNull(input);

        // load the properties file
        this.properties.load(input);


      } catch (IOException e) {
        LOGGER.error("Error while loading properties file from: " + PROPERTIES_PATH, e);

      }finally {
        if (input != null) {
          try {
            input.close();
          } catch (IOException e) {
            LOGGER.error("Error while closing file input stream", e);
          }
        }
      }

    }

    @Override
    public Properties getProperties() {
      return properties;
    }
  };

  //path to property file
  private static  final String PROPERTIES_PATH = "/WEB-INF/workloadAPI.properties";
  //property keys
  public static final String WORKLOAD_BINARIES_FOLDER = "workloadBinariesFolder";
  public static final String WORKLOAD_RESULTS_FOLDER = "workloadResultsFolder";
  public static final String WORKLOAD_DATA_FOLDER = "workloadDataFolder";

  private  static final Logger LOGGER = LoggerFactory.getLogger(WorkloadProcess.class.getName());

  public static WorkloadApiProperties getInstance() {
    return WorkloadApiProperties.INSTANCE;
  }
}
