package de.uulm.omi.workload.tpcc;

import static com.google.common.base.Preconditions.checkNotNull;

import de.uulm.omi.cloudiator.dbms.evaluator.model.DBEndpoint;
import de.uulm.omi.cloudiator.dbms.evaluator.model.DBEndpoints;
import de.uulm.omi.cloudiator.dbms.evaluator.model.TPCCWorkloadConfig;
import java.util.StringJoiner;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Created by Daniel Seybold on 06.09.2018.
 */
public class TPCCCommandBuilder {

  private static final Logger LOGGER = LoggerFactory.getLogger(TPCCCommandBuilder.class);

  public static String buildStartCommand(TPCCWorkloadConfig workload, DBEndpoints dbEndpoints,
      String tpccBinary) {

    checkNotNull(workload);
    checkNotNull(dbEndpoints);
    checkNotNull(tpccBinary);

    String startCommand = tpccBinary + " workload run tpcc ";

    startCommand += buildWorklaodCommand(workload);

    startCommand += buildConnectionUrl(dbEndpoints);

    return startCommand;
  }

  public static String buildInitCommand(DBEndpoints dbEndpoints, String tpccBinary){

    String initCommand = tpccBinary + " workload init tpcc ";

    initCommand += buildConnectionUrl(dbEndpoints);

    return  initCommand;
  }


  private static String buildWorklaodCommand(TPCCWorkloadConfig workload){

    String workloadCommand = "";

    if (workload.getActiveWarehouses().intValue() > 0){
      workloadCommand += " --active-warehouses " + workload.getActiveWarehouses().intValue();
    }

    if (workload.isDrop()){
      workloadCommand += " --drop ";
    }

    if (workload.getDuration().intValue() > 0){
      workloadCommand += " --duration " + workload.getDuration().intValue() + "s ";
    }

    if (workload.isExpensiveChecks()){
      workloadCommand += " --expensive-checks ";
    }

    if (!workload.getHistogram().isEmpty()){
      workloadCommand += " --histograms \"" + workload.getHistogram() + ".txt\" ";
    }

    //TODO: remove for the future as it will not be supported for CockroachDB 2.0
    if (workload.isInit()){
      workloadCommand += " --init ";
    }


    if (workload.isInterleaved()){
      workloadCommand += " --interleaved ";
    }

    if (workload.getMaxOps().intValue() > 0){
      workloadCommand += " --max-ops " + workload.getMaxOps().intValue();
    }

    if (workload.getMaxRate().floatValue() > 0){
      workloadCommand += " --max-rate " + workload.getMaxRate().floatValue();
    }

    if (!workload.getOpsMix().isEmpty()){
      workloadCommand += " --mix \"" + workload.getOpsMix() + "\" ";
    }

    if (workload.getPartitionAffinity().intValue() > 0){
      workloadCommand += " --partition-affinity " + workload.getPartitionAffinity().intValue();
    }

    if (workload.getPartitions().intValue() > 0){
      workloadCommand += " --partitions " + workload.getPartitions().intValue();
    }

    if (workload.getRamp().intValue() > 0){
      workloadCommand += " --ramp " + workload.getRamp().intValue() + "s ";
    }

    if (workload.isScatter()){
      workloadCommand += " --scatter ";
    }

    if (workload.getSeed().intValue() > 0){
      workloadCommand += " --seed " + workload.getSeed().intValue();
    }

    if (workload.isSerializable()){
      workloadCommand += " --serializable ";
    }

    if (workload.isSplit()){
      workloadCommand += " --split ";
    }

    if (workload.isTolerateErrors()){
      workloadCommand += " --tolerate-errors ";
    }

    //TODO: needs to be changed to expect a number!
    /*
    if (workload.isWait()){
      workloadCommand += " --wait ";
    }

     */

    if (workload.getWarehouses().intValue() > 0){
      workloadCommand += " --warehouses " + workload.getWarehouses().intValue();
    }

    if (workload.getWorkers().intValue() > 0){
      workloadCommand += " --workers " + workload.getWorkers().intValue();
    }

    if (!workload.getZones().isEmpty()){
      workloadCommand += " --zones \"" + workload.getZones() + "\" ";
    }

    return workloadCommand;
  }

  private static String buildConnectionUrl(DBEndpoints dbEndpoints){

    String connectionUrl;

    StringJoiner dbmsEndpoints = new StringJoiner(" ");
    for(DBEndpoint endpoint : dbEndpoints){

      String postgresEndpoint;

      if(endpoint.getPort().intValue() != 0){
        postgresEndpoint = endpoint.getIpAddress() + ":" + endpoint.getPort().intValue();
      }else{
        postgresEndpoint = endpoint.getIpAddress() + ":26257";
      }

      String postgresUrl =  "\"postgresql://" + postgresEndpoint+"?sslmode=disable\"";

      dbmsEndpoints.add(postgresUrl);

    }
    connectionUrl = " " + dbmsEndpoints.toString();

    LOGGER.debug("DBMS connection URL for TPC-C is: " + connectionUrl);

    return connectionUrl;
  }
}
